package com.halomobi.view.imageloader;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;


import com.halomobi.network.net.AbsNetTask;
import com.halomobi.network.net.http.HttpResult;
import com.halomobi.network.schedule.ThreadManager;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.KeyUtil;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.Utils;
import com.halomobi.view.gif.GifView;
import com.halomobi.view.imageloader.cache.DoubleCache;
import com.halomobi.view.imageloader.cache.ImageCache;

import java.io.InputStream;
import java.net.HttpURLConnection;

@SuppressWarnings("deprecation")
public class ImageLoader {
    /**
     * 图片下载的线程池名称
     */
    private static final String THREAD_POOL_NAME = "IMAGE_THREAD_POOL";
    private static BitmapFactory.Options mOptions = new BitmapFactory.Options();
    /**
     * 图片下载的线程池
     */
    private static ThreadManager.ThreadPoolProxy mThreadPool = ThreadManager.getSinglePool(THREAD_POOL_NAME);
    private static ImageCache imageCache = new DoubleCache();

//	private static volatile ImageLoader instance;DownloadDBManager

    static {
        mOptions.inDither = false;// 设置为false，将不考虑图片的抖动值，这会减少图片的内存占用
        mOptions.inPurgeable = true;// 设置为true，表示允许系统在内存不足时，删除bitmap的数组。
        mOptions.inInputShareable = true;// 和inPurgeable配合使用，如果inPurgeable是false，那么该参数将被忽略，表示是否对bitmap的数组进行共享
    }

    /**
     * 加载图片
     */
    public static void load(View view, String url) {
        if (view == null || TextUtils.isEmpty(url)) {
            return;
        }
        view.setTag(url);// 把控件和图片的url进行绑定，因为加载是一个耗时的，等加载完毕了需要判定该控件是否和该url匹配
        byte[] data = imageCache.getCache(url);
        if (data != null) {
            setImageSafe(view, url, data);// 如果内存中加载到了，直接设置图片
        } else {
            asyncLoad(view, url);
        }
    }

    /**
     * 异步加载图片
     */

    private static void asyncLoad(final View view, final String url) {
        // 先创建一个runnable封装执行过程
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                byte[] data = loadFromNet(url);// 如果内存中加载到了，直接设置图片
                if (data != null && data.length != 0) {
                    setImageSafe(view, url, data);
                }
            }
        };
        mThreadPool.execute(runnable);// 执行任务
    }

    public static Bitmap getBitmap(String url) {
        Bitmap bitmap = null;
        byte[] array = getByteArray(url);
        if (array != null && array.length > 0) {
            bitmap = BitmapFactory.decodeByteArray(array, 0, array.length);
        }
        return bitmap;
    }

    public static byte[] getByteArray(String url) {
        byte[] data = new byte[0];
        try {
            if (!TextUtils.isEmpty(url)) {
                data = imageCache.getCache(url);
                if (data == null || data.length <= 0) {
                    data = loadFromNet(url);// 如果内存中加载到了，直接设置图片
                }
                return data;
            }
        } catch (OutOfMemoryError e) {
            imageCache.clear();
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return data;
    }

    /**
     * 从网络加载图片
     */
    private static byte[] loadFromNet(final String url) {
        byte[] data = new byte[0];
        long curTime = System.currentTimeMillis();
        LogUtils.i("start connect ");
        HttpResult httpResult = null;
        try {
            httpResult = new HttpResult(new AbsNetTask(AbsNetTask.ReqType.Get) {

                @Override
                public LaunchMode getLaunchMode() {
                    return LaunchMode.ADD_NEW;
                }

                @Override
                public String getName() {
                    return "ImageLoader";
                }

                @Override
                protected byte[] getEntity() {
                    return new byte[0];
                }

                @Override
                protected String getUrl() throws Exception {
                    return url;
                }

                @Override
                protected void onAddHeaders(HttpURLConnection urlConnection) {

                }
            }.openConnection(url));
        } catch (Exception e) {
            e.printStackTrace();
        }
        LogUtils.i("end connect cast " + (System.currentTimeMillis() - curTime));
        InputStream stream;
        if (httpResult != null && (stream = httpResult.getInputStream()) != null) {// 请求网络
            curTime = System.currentTimeMillis();
            LogUtils.i("start loadImageDate ");
            data = FileUtils.getByteFromStream(stream);
            LogUtils.i("end loadImageDate cast " + (System.currentTimeMillis() - curTime));
            httpResult.close();
            if (data != null && data.length > 0) {
                imageCache.putCache(url, data);
            }
        }
        return data;
    }

    public static Bitmap adapterData(View view, byte[] data) {
        Bitmap bitmap = null;
        byte[] dst = new byte[4];
        System.arraycopy(data, 0, dst, 0, 4);
        String fileType = KeyUtil.bytesToHexString(dst);
        if (!fileType.equalsIgnoreCase("47494638") || view.getClass() != GifView.class) {// 非gif数据或者非gif渲染控件
            bitmap = BitmapFactory.decodeByteArray(data, 0, data.length, mOptions);
        }
        return bitmap;
    }

    /**
     * 设置给控件图片
     */
    @SuppressLint("NewApi")
    private static void setImageSafe(final View view, final String url,
                                     final byte[] data) {
        if (view == null || view.getTag() == null || TextUtils.isEmpty(url)
                || data == null || data.length == 0) {
            return;
        }

        final Bitmap bitmap = adapterData(view, data);
        // 检测文件头
        Utils.runInMainThread(new Runnable() {// 需要在主线程中设置
            @Override
            public void run() {
                Object tag;// 在主线程中判断，可以做到同步
                if ((tag = view.getTag()) != null && tag instanceof String) {
                    String str = (String) tag;
                    if (TextUtils.equals(str, url)) {// 检测如果url和控件匹配
                        int sdk = android.os.Build.VERSION.SDK_INT;
                        if (bitmap != null) {
                            if (view instanceof ImageView) {
                                ((ImageView) view).setImageBitmap(bitmap);
                                return;
                            }

                            if (sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
                                view.setBackgroundDrawable(new BitmapDrawable(bitmap));
                            } else {
                                view.setBackground(new BitmapDrawable(bitmap));// 就进行图片设置
                            }

                        } else if (view instanceof GifView) {
                            GifView gifView = (GifView) view;
                            gifView.setShowDimension(gifView.getMeasuredWidth(), gifView.getMeasuredHeight());
                            gifView.setGifImage(data);
                        }

                    }
                }
            }

        });
    }

}
